# -*- coding: utf-8 -*-
#
# Copyright (C) 2014 GNS3 Technologies Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""
Base class (interface) for modules.
"""

from ..qt import QtCore
from ..local_config import LocalConfig

import logging
log = logging.getLogger(__name__)


class Module(QtCore.QObject):

    """
    Module interface.
    """

    notification_signal = QtCore.Signal(str, str)

    def __init__(self):

        super().__init__()
        LocalConfig.instance().config_changed_signal.connect(self.configChangedSlot)

    def configChangedSlot(self):
        """
        Call when the configuration file has changed
        """

        raise NotImplementedError("Missing configChangedSlot in {}".format(self.__class__.__name__))

    @staticmethod
    def nodes(self):
        """
        Returns all nodes supported by this module.
        Must be overloaded.

        :returns: list of node classes
        """

        raise NotImplementedError()

    @staticmethod
    def preferencePages():
        """
        Returns all the preference pages used by this module.
        Must be overloaded.

        :returns: list of preference page classes
        """

        raise NotImplementedError()



# -*- coding: utf-8 -*-
#
# Copyright (C) 2015 GNS3 Technologies Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""Wizard for Docker images."""

from gns3.qt import QtGui, QtWidgets
from gns3.dialogs.vm_wizard import VMWizard

from ..ui.docker_vm_wizard_ui import Ui_DockerVMWizard
from .. import Docker


class DockerVMWizard(VMWizard, Ui_DockerVMWizard):
    """Wizard to create a Docker image.

    :param docker_images: existing Docker images
    :param parent: parent widget
    """

    def __init__(self, docker_images, parent):

        super().__init__(parent=parent, devices=[], use_local_server=True)
        self.setPixmap(QtWidgets.QWizard.LogoPixmap, QtGui.QPixmap(
            ":/icons/docker.png"))
        self.setWizardStyle(QtWidgets.QWizard.ModernStyle)

        self._docker_images = docker_images

        if Docker.instance().settings()["use_local_server"]:
            # skip the server page if we use the local server
            self.setStartId(1)

    def initializePage(self, page_id):

        super().initializePage(page_id)

        if self.page(page_id) == self.uiImageWizardPage:
            self._server.get(
                "/docker/images", self._getDockerImagesFromServerCallback)

    def _getDockerImagesFromServerCallback(
            self, result, error=False, **kwargs):
        """Callback for getDockerImagesFromServer.

        :param progress_dialog: QProgressDialog instance
        :param result: server response
        :param error: indicates an error (boolean)
        """
        if error:
            QtWidgets.QMessageBox.critical(
                self, "Docker Images", "{}".format(result["message"]))
        else:
            self.uiImageListComboBox.clear()
            existing_images = []
            for existing_image in self._docker_images.values():
                existing_images.append(existing_image["imagename"])

            for image in result:
                if image["imagename"] not in existing_images:
                    self.uiImageListComboBox.addItem(image["imagename"], image)

    def validateCurrentPage(self):
        """Validates the server."""

        if super().validateCurrentPage() is False:
            return False

        if self.currentPage() == self.uiImageWizardPage:
            if not self.uiImageListComboBox.count():
                QtWidgets.QMessageBox.critical(
                    self, "Docker images",
                    "There are no Docker images available!")
                return False
        return True

    def getSettings(self):
        """Returns the settings set in this Wizard.

        :return: settings
        :rtype: dict
        """

        if self.uiLocalRadioButton.isChecked():
            server = "local"
        elif self.uiRemoteRadioButton.isChecked():
            server = self.uiRemoteServersComboBox.currentText()
        elif self.uiVMRadioButton.isChecked():
            server = "vm"

        index = self.uiImageListComboBox.currentIndex()
        imagename = self.uiImageListComboBox.itemText(index)
        # FIXME: add some more configuration options for images
        imageinfo = self.uiImageListComboBox.itemData(index)

        settings = {
            "imagename": imagename,
            "server": server,
        }
        return settings



# -*- coding: utf-8 -*-
#
# Copyright (C) 2014 GNS3 Technologies Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""
NIO implementation on the client side (in the form of a pseudo node represented as a cloud).
"""

import re
from gns3.node import Node
from gns3.ports.port import Port
from gns3.nios.nio_generic_ethernet import NIOGenericEthernet
from gns3.nios.nio_linux_ethernet import NIOLinuxEthernet
from gns3.nios.nio_nat import NIONAT
from gns3.nios.nio_udp import NIOUDP
from gns3.nios.nio_tap import NIOTAP
from gns3.nios.nio_unix import NIOUNIX
from gns3.nios.nio_vde import NIOVDE
from gns3.nios.nio_null import NIONull

import logging
log = logging.getLogger(__name__)


class Cloud(Node):

    """
    Dynamips cloud.

    :param module: parent module for this node
    :param server: GNS3 server instance
    :param project: Project instance
    """

    _name_instance_count = 1

    def __init__(self, module, server, project):

        super().__init__(module, server, project)

        log.info("cloud is being created")
        # create an unique id and name
        self._name_id = Cloud._name_instance_count
        Cloud._name_instance_count += 1

        name = "Cloud {}".format(self._name_id)
        self.setStatus(Node.started)  # this is an always-on node
        self._initial_settings = None
        self._settings = {"name": name,
                          "interfaces": {},
                          "nios": []}

    def delete(self):
        """
        Deletes this cloud.
        """

        # first delete all the links attached to this node
        self.delete_links_signal.emit()
        self.deleted_signal.emit()

    def setup(self, name=None, additional_settings={}):
        """
        Setups this cloud.

        :param name: optional name for this cloud
        """

        if name:
            self._settings["name"] = name

        if additional_settings and "nios" in additional_settings:
            self._settings["nios"] = additional_settings["nios"]

        self._server.get("/interfaces", self._setupCallback)

    def _setupCallback(self, result, error=False, **kwargs):
        """
        Callback for setup.

        :param result: server response
        :param error: indicates an error (boolean)
        """

        if error:
            log.error("error while setting up {}: {}".format(self.name(), result["message"]))
            # a warning message instead of a error is more appropriate here
            self.warning_signal.emit(self.id(), result["message"])
        else:
            self._settings["interfaces"] = result.copy()

        if self._settings["nios"]:
            self._addPorts(self._settings["nios"])

        if self._loading:
            self.loaded_signal.emit()
        else:
            self.setInitialized(True)
            log.info("cloud {} has been created".format(self.name()))
            self.created_signal.emit(self.id())
            self._module.addNode(self)

    def _createNIOUDP(self, nio):
        """
        Creates a NIO UDP.

        :param nio: nio string
        """

        match = re.search(r"""^nio_udp:(\d+):(.+):(\d+)$""", nio)
        if match:
            lport = int(match.group(1))
            rhost = match.group(2)
            rport = int(match.group(3))
            return NIOUDP(lport, rhost, rport)
        return None

    def _createNIOGenericEthernet(self, nio):
        """
        Creates a NIO Generic Ethernet.

        :param nio: nio string
        """

        match = re.search(r"""^nio_gen_eth:(.+)$""", nio)
        if match:
            ethernet_device = match.group(1)
            return NIOGenericEthernet(ethernet_device)
        return None

    def _createNIOLinuxEthernet(self, nio):
        """
        Creates a NIO Linux Ethernet.

        :param nio: nio string
        """

        match = re.search(r"""^nio_gen_linux:(.+)$""", nio)
        if match:
            linux_device = match.group(1)
            return NIOLinuxEthernet(linux_device)
        return None

    def _createNIONAT(self, nio):
        """
        Creates a NIO NAT.

        :param nio: nio string
        """

        match = re.search(r"""^nio_nat:(.+)$""", nio)
        if match:
            identifier = match.group(1)
            return NIONAT(identifier)
        return None

    def _createNIOTAP(self, nio):
        """
        Creates a NIO TAP.

        :param nio: nio string
        """

        match = re.search(r"""^nio_tap:(.+)$""", nio)
        if match:
            tap_device = match.group(1)
            return NIOTAP(tap_device)
        return None

    def _createNIOUNIX(self, nio):
        """
        Creates a NIO UNIX.

        :param nio: nio string
        """

        match = re.search(r"""^nio_unix:(.+):(.+)$""", nio)
        if match:
            local_file = match.group(1)
            remote_file = match.group(2)
            return NIOUNIX(local_file, remote_file)
        return None

    def _createNIOVDE(self, nio):
        """
        Creates a NIO VDE.

        :param nio: nio string
        """

        match = re.search(r"""^nio_vde:(.+):(.+)$""", nio)
        if match:
            control_file = match.group(1)
            local_file = match.group(2)
            return NIOVDE(control_file, local_file)
        return None

    def _createNIONull(self, nio):
        """
        Creates a NIO Null.

        :param nio: nio string
        """

        match = re.search(r"""^nio_null:(.+)$""", nio)
        if match:
            identifier = match.group(1)
            return NIONull(identifier)
        return None

    def _allocateNIO(self, nio):
        """
        Allocate a new NIO object.

        :param nio: NIO description

        :returns: NIO instance
        """

        nio_object = None
        if nio.lower().startswith("nio_udp"):
            nio_object = self._createNIOUDP(nio)
        if nio.lower().startswith("nio_gen_eth"):
            nio_object = self._createNIOGenericEthernet(nio)
        if nio.lower().startswith("nio_gen_linux"):
            nio_object = self._createNIOLinuxEthernet(nio)
        if nio.lower().startswith("nio_nat"):
            nio_object = self._createNIONAT(nio)
        if nio.lower().startswith("nio_tap"):
            nio_object = self._createNIOTAP(nio)
        if nio.lower().startswith("nio_unix"):
            nio_object = self._createNIOUNIX(nio)
        if nio.lower().startswith("nio_vde"):
            nio_object = self._createNIOVDE(nio)
        if nio.lower().startswith("nio_null"):
            nio_object = self._createNIONull(nio)
        if nio_object is None:
            log.error("Could not create NIO object from {}".format(nio))
        return nio_object

    def _addPorts(self, nios, ignore_existing_nio=False):
        """
        Adds adapters.

        :param adapters: number of adapters
        """

        # add ports
        for nio in nios:
            if ignore_existing_nio and nio in self._settings["nios"]:
                # port already created for this NIO
                continue
            nio_object = self._allocateNIO(nio)
            if nio_object is None:
                continue
            port = Port(nio, nio_object, stub=True)
            port.setStatus(Port.started)
            self._ports.append(port)
            log.debug("port {} has been added".format(nio))

    def update(self, new_settings):
        """
        Updates the settings for this cloud.

        :param new_settings: settings dictionary
        """

        updated = False
        if "nios" in new_settings:
            nios = new_settings["nios"]
            self._addPorts(nios, ignore_existing_nio=True)
            updated = True

            # delete ports
            for nio in self._settings["nios"]:
                if nio not in nios:
                    for port in self._ports.copy():
                        if port.name() == nio:
                            self._ports.remove(port)
                            updated = True
                            log.debug("port {} has been deleted".format(nio))
                            break

            self._settings["nios"] = new_settings["nios"].copy()

        if "name" in new_settings and new_settings["name"] != self.name():
            self._settings["name"] = new_settings["name"]
            updated = True

        if updated:
            log.info("cloud {} has been updated".format(self.name()))
            self.updated_signal.emit()

    def deleteNIO(self, port):

        pass

    def info(self):
        """
        Returns information about this cloud.

        :returns: formated string
        """

        info = """Cloud device {name} is always-on
This is a pseudo-device for external connections
""".format(name=self.name())

        port_info = ""
        for port in self._ports:
            if port.isFree():
                port_info += "   Port {} is empty\n".format(port.name())
            else:
                port_info += "   Port {name} {description}\n".format(name=port.name(),
                                                                     description=port.description())

            # add the Windows interface name
            match = re.search(r"""^nio_gen_eth:(\\device\\npf_.+)$""", port.name())
            if match:
                for interface in self._settings["interfaces"]:
                    if interface["name"].lower() == match.group(1):
                        port_info += "      Windows name: {}\n".format(interface["description"])
                        break

        return info + port_info

    def dump(self):
        """
        Returns a representation of this cloud
        (to be saved in a topology file).

        :returns: representation of the node (dictionary)
        """

        cloud = {"id": self.id(),
                 "type": self.__class__.__name__,
                 "description": str(self),
                 "properties": {"name": self.name(),
                                "nios": self._settings["nios"]},
                 "server_id": self._server.id()}

        # add the ports
        if self._ports:
            ports = cloud["ports"] = []
            for port in self._ports:
                ports.append(port.dump())

        return cloud

    def load(self, node_info):
        """
        Loads a cloud representation
        (from a topology file).

        :param node_info: representation of the node (dictionary)
        """

        settings = node_info["properties"]
        name = settings.pop("name")
        log.info("cloud {} is loading".format(name))
        self.setName(name)
        self._loading = True
        self._node_info = node_info
        self.loaded_signal.connect(self._updatePortSettings)
        self.setup(name, additional_settings=settings)

    def _updatePortSettings(self):
        """
        Updates port settings when loading a topology.
        """

        self.loaded_signal.disconnect(self._updatePortSettings)

        # update the port with the correct IDs
        if "ports" in self._node_info:
            ports = self._node_info["ports"]
            for topology_port in ports:
                for port in self._ports:
                    if topology_port["name"] == port.name():
                        port.setId(topology_port["id"])
                        if topology_port["name"].startswith("nio_gen_eth") or topology_port["name"].startswith("nio_linux_eth"):
                            # lookup if the interface exists
                            available_interface = False
                            topology_port_name = topology_port["name"].split(':', 1)[1]
                            for interface in self._settings["interfaces"]:
                                if interface["name"] == topology_port_name:
                                    available_interface = True
                                    break
                            if not available_interface:
                                alternative_interface = self._module.findAlternativeInterface(self, topology_port_name)
                                if alternative_interface:
                                    if topology_port["name"] in self._settings["nios"]:
                                        self._settings["nios"].remove(topology_port["name"])
                                    topology_port["name"] = topology_port["name"].replace(topology_port_name, alternative_interface)
                                    nio = self._allocateNIO(topology_port["name"])
                                    port.setDefaultNio(nio)
                                    port.setName(topology_port["name"])
                                    self._settings["nios"].append(topology_port["name"])

        # now we can set the node as initialized and trigger the created signal
        self.setInitialized(True)
        log.info("cloud {} has been loaded".format(self.name()))
        self.created_signal.emit(self.id())
        self._module.addNode(self)
        self._loading = False
        self._node_info = None

    def name(self):
        """
        Returns the name of this cloud.

        :returns: name (string)
        """

        return self._settings["name"]

    def settings(self):
        """
        Returns all this cloud settings.

        :returns: settings dictionary
        """

        return self._settings

    def ports(self):
        """
        Returns all the ports for this cloud.

        :returns: list of Port instances
        """

        return self._ports

    def configPage(self):
        """
        Returns the configuration page widget to be used by the node properties dialog.

        :returns: QWidget object
        """

        from .pages.cloud_configuration_page import CloudConfigurationPage
        return CloudConfigurationPage

    @staticmethod
    def defaultSymbol():
        """
        Returns the default symbol path for this cloud.

        :returns: symbol path (or resource).
        """

        return ":/symbols/cloud.svg"

    @staticmethod
    def symbolName():

        return "Cloud"

    @staticmethod
    def categories():
        """
        Returns the node categories the node is part of (used by the device panel).

        :returns: list of node category (integer)
        """

        return [Node.end_devices]

    def __str__(self):

        return "Cloud"



# -*- coding: utf-8 -*-
#
# Copyright (C) 2014 GNS3 Technologies Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import os
import pathlib
import glob

from gns3.servers import Servers
from gns3.qt import QtWidgets
from gns3.utils.file_copy_worker import FileCopyWorker
from gns3.utils.progress_dialog import ProgressDialog


class ImageManager:

    def __init__(self):
        # Remember if we already ask the user about this image for this server
        self._asked_for_this_image = {}

    def askCopyUploadImage(self, parent, path, server, vm_type):
        """
        Ask user for copying the image to the default directory or upload
        it to remote server.

        :param parent: Parent window
        :param path: File path on computer
        :param server: The server where the images should be located
        :param vm_type: Remote upload endpoint
        :returns path: Final path
        """

        if server and not server.isLocal():
            return self._uploadImageToRemoteServer(path, server, vm_type)
        else:
            destination_directory = self.getDirectoryForType(vm_type)
            if os.path.normpath(os.path.dirname(path)) != destination_directory:
                # the IOS image is not in the default images directory
                reply = QtWidgets.QMessageBox.question(parent,
                                                       'Image',
                                                       'Would you like to copy {} to the default images directory'.format(os.path.basename(path)),
                                                       QtWidgets.QMessageBox.Yes,
                                                       QtWidgets.QMessageBox.No)
                if reply == QtWidgets.QMessageBox.Yes:
                    destination_path = os.path.join(destination_directory, os.path.basename(path))
                    try:
                        os.makedirs(destination_directory, exist_ok=True)
                    except OSError as e:
                        QtWidgets.QMessageBox.critical(parent, 'Image', 'Could not create destination directory {}: {}'.format(destination_directory, str(e)))
                        return path
                    worker = FileCopyWorker(path, destination_path)
                    progress_dialog = ProgressDialog(worker, 'Image', 'Copying {}'.format(os.path.basename(path)), 'Cancel', busy=True, parent=parent)
                    progress_dialog.show()
                    progress_dialog.exec_()
                    errors = progress_dialog.errors()
                    if errors:
                        QtWidgets.QMessageBox.critical(parent, 'Image', '{}'.format(''.join(errors)))
                        return path
                    else:
                        path = destination_path
            return path

    def _uploadImageToRemoteServer(self, path, server, vm_type):
        """
        Upload image to remote server

        :param path: File path on computer
        :param server: The server where the images should be located
        :param vm_type: Image vm_type
        :returns path: Final path
        """

        if vm_type == 'QEMU':
            upload_endpoint = '/qemu/vms'
        elif vm_type == 'IOU':
            upload_endpoint = '/iou/vms'
        elif vm_type == 'DYNAMIPS':
            upload_endpoint = '/dynamips/vms'
        else:
            raise Exception('Invalid image vm_type')

        filename = self._getRelativeImagePath(path, vm_type).replace("\\", "/")
        server.post('{}/{}'.format(upload_endpoint, filename), None, body=pathlib.Path(path), progressText="Uploading {}".format(filename))
        return filename

    def addMissingImage(self, filename, server, vm_type):
        """
        Add a missing image to the queue of images require to be upload on remote server
        :param filename: Filename of the image
        :param server: Server where image should be uploaded
        :param vm_type: Type of the image
        """

        if self._asked_for_this_image.setdefault(server.id(), {}).setdefault(filename, False):
            return
        self._asked_for_this_image[server.id()][filename] = True

        if server.isLocal():
            return
        path = os.path.join(self.getDirectoryForType(vm_type), filename)
        if os.path.exists(path):
            if self._askForUploadMissingImage(filename, server):

                if filename.endswith(".vmdk"):
                    # A vmdk file could be split in multiple vmdk file
                    search = glob.escape(path).replace(".vmdk", "-*.vmdk")
                    for file in glob.glob(search):
                        self._uploadImageToRemoteServer(file, server, vm_type)

                self._uploadImageToRemoteServer(path, server, vm_type)
                del self._asked_for_this_image[server.id()][filename]

    def _askForUploadMissingImage(self, filename, server):
        from gns3.main_window import MainWindow
        parent = MainWindow.instance()
        reply = QtWidgets.QMessageBox.warning(parent,
                                              'Image',
                                              '{} is missing on server {} but exist on your computer. Do you want to upload it?'.format(filename, server.url()),
                                              QtWidgets.QMessageBox.Yes,
                                              QtWidgets.QMessageBox.No)
        if reply == QtWidgets.QMessageBox.Yes:
            return True
        return False

    def _getRelativeImagePath(self, path, vm_type):
        """
        Get a path relative to images directory path
        or just filename if the path is not located inside
        image directory

        :param path: file path
        :param vm_type: Type of vm
        :return: file path
        """

        if not path:
            return ""
        img_directory = self.getDirectoryForType(vm_type)
        path = os.path.abspath(path)
        if os.path.commonprefix([img_directory, path]) == img_directory:
            return os.path.relpath(path, img_directory)
        return os.path.basename(path)

    def getDirectory(self):
        """
        Returns the images directory path.

        :returns: path to the default images directory
        """

        return Servers.instance().localServerSettings()['images_path']

    def getDirectoryForType(self, vm_type):
        """
        Return the path of local directory of the images
        of a specific vm_type

        :param vm_type: Type of vm
        """
        if vm_type == 'DYNAMIPS':
            return os.path.join(self.getDirectory(), 'IOS')
        else:
            return os.path.join(self.getDirectory(), vm_type)

    @staticmethod
    def instance():
        """
        Singleton to return only on instance of ImageManager.

        :returns: instance of ImageManager
        """

        if not hasattr(ImageManager, '_instance') or ImageManager._instance is None:
            ImageManager._instance = ImageManager()
        return ImageManager._instance



# -*- coding: utf-8 -*-
#
# Copyright (C) 2015 GNS3 Technologies Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""
QEMU VM implementation.
"""

from gns3.vm import VM
from gns3.node import Node
from gns3.image_manager import ImageManager
from gns3.ports.port import Port
from gns3.ports.ethernet_port import EthernetPort
from .settings import QEMU_VM_SETTINGS


import logging
log = logging.getLogger(__name__)


class QemuVM(VM):

    """
    QEMU VM.

    :param module: parent module for this node
    :param server: GNS3 server instance
    """
    URL_PREFIX = "qemu"

    def __init__(self, module, server, project):
        super().__init__(module, server, project)

        log.info("QEMU VM instance is being created")
        self._port_name_format = None
        self._port_segment_size = 0
        self._first_port_name = None
        self._linked_clone = True

        self._settings = {"name": "",
                          "usage": "",
                          "qemu_path": "",
                          "hda_disk_image": "",
                          "hdb_disk_image": "",
                          "hdc_disk_image": "",
                          "hdd_disk_image": "",
                          "hda_disk_interface": QEMU_VM_SETTINGS["hda_disk_interface"],
                          "hdb_disk_interface": QEMU_VM_SETTINGS["hdb_disk_interface"],
                          "hdc_disk_interface": QEMU_VM_SETTINGS["hdc_disk_interface"],
                          "hdd_disk_interface": QEMU_VM_SETTINGS["hdd_disk_interface"],
                          "cdrom_image": "",
                          "hda_disk_image_md5sum": "",
                          "hdb_disk_image_md5sum": "",
                          "hdc_disk_image_md5sum": "",
                          "hdd_disk_image_md5sum": "",
                          "cdrom_image_md5sum": "",
                          "boot_priority": QEMU_VM_SETTINGS["boot_priority"],
                          "options": "",
                          "ram": QEMU_VM_SETTINGS["ram"],
                          "cpus": QEMU_VM_SETTINGS["cpus"],
                          "console": None,
                          "console_type": QEMU_VM_SETTINGS["console_type"],
                          "adapters": QEMU_VM_SETTINGS["adapters"],
                          "adapter_type": QEMU_VM_SETTINGS["adapter_type"],
                          "mac_address": QEMU_VM_SETTINGS["mac_address"],
                          "legacy_networking": QEMU_VM_SETTINGS["legacy_networking"],
                          "platform": QEMU_VM_SETTINGS["platform"],
                          "acpi_shutdown": QEMU_VM_SETTINGS["acpi_shutdown"],
                          "cpu_throttling": QEMU_VM_SETTINGS["cpu_throttling"],
                          "process_priority": QEMU_VM_SETTINGS["process_priority"],
                          "initrd": "",
                          "kernel_image": "",
                          "initrd_md5sum": "",
                          "kernel_image_md5sum": "",
                          "kernel_command_line": ""}

    def _addAdapters(self, adapters):
        """
        Adds adapters.

        :param adapters: number of adapters
        """

        interface_number = segment_number = 0
        for adapter_number in range(0, adapters):
            if self._first_port_name and adapter_number == 0:
                port_name = self._first_port_name
            else:
                port_name = self._port_name_format.format(
                    interface_number,
                    segment_number,
                    port0 = interface_number,
                    port1 = 1 + interface_number,
                    segment0 = segment_number,
                    segment1 = 1 + segment_number
                )
                interface_number += 1
                if self._port_segment_size and interface_number % self._port_segment_size == 0:
                    segment_number += 1
                    interface_number = 0
            new_port = EthernetPort(port_name)
            new_port.setAdapterNumber(adapter_number)
            new_port.setPortNumber(0)
            new_port.setHotPluggable(False)
            self._ports.append(new_port)
            log.debug("Adapter {} with port {} has been added".format(adapter_number, port_name))

    def setup(self, qemu_path, name=None, vm_id=None, port_name_format="Ethernet{0}",
              port_segment_size=0, first_port_name="", linked_clone=True, additional_settings={}, base_name=None):
        """
        Setups this QEMU VM.

        :param name: optional name
        :param vm_id: VM identifier
        """

        # let's create a unique name if none has been chosen
        if not name:
            if linked_clone:
                name = self.allocateName(base_name + "-")
            else:
                name = base_name
                self.setName(name)

        if not name:
            self.error_signal.emit(self.id(), "could not allocate a name for this QEMU VM")
            return

        self._settings["name"] = name
        self._linked_clone = linked_clone
        params = {"name": name,
                  "qemu_path": qemu_path,
                  "linked_clone": linked_clone}

        if vm_id:
            params["vm_id"] = vm_id

        self._port_name_format = port_name_format
        self._port_segment_size = port_segment_size
        self._first_port_name = first_port_name
        params.update(additional_settings)
        self.httpPost("/qemu/vms", self._setupCallback, body=params, progressText="Creating {}".format(name))

    def _setupCallback(self, result, error=False, **kwargs):
        """
        Callback for setup.

        :param result: server response
        :param error: indicates an error (boolean)
        """

        if not super()._setupCallback(result, error=error, **kwargs):
            return

        # create the ports on the client side
        self._addAdapters(self._settings.get("adapters", 0))

        if self._loading:
            self.loaded_signal.emit()
        else:
            self.setInitialized(True)
            log.info("QEMU VM instance {} has been created".format(self.name()))
            self.created_signal.emit(self.id())
            self._module.addNode(self)

        for image_field in ["hda_disk_image", "hdb_disk_image", "hdc_disk_image", "hdd_disk_image", "initrd", "kernel_image", "cdrom_image"]:
            if image_field in result and result[image_field] is not None and result[image_field] != "":
                # The image is missing on remote server
                field = "{}_md5sum".format(image_field)
                if field not in result or result[field] is None or len(result[field]) == 0:
                    ImageManager.instance().addMissingImage(result[image_field], self._server, "QEMU")

    def update(self, new_settings):
        """
        Updates the settings for this QEMU VM.

        :param new_settings: settings dictionary
        """

        if "name" in new_settings and new_settings["name"] != self.name() and self.hasAllocatedName(new_settings["name"]):
            self.error_signal.emit(self.id(), 'Name "{}" is already used by another node'.format(new_settings["name"]))
            return

        params = {}
        for name, value in new_settings.items():
            if name in self._settings and self._settings[name] != value:
                params[name] = value

        log.debug("{} is updating settings: {}".format(self.name(), params))
        self.httpPut("/qemu/vms/{vm_id}".format(vm_id=self._vm_id), self._updateCallback, body=params)

    def _updateCallback(self, result, error=False, **kwargs):
        """
        Callback for update.

        :param result: server response
        :param error: indicates an error (boolean)
        """

        if error:
            log.error("error while deleting {}: {}".format(self.name(), result["message"]))
            self.server_error_signal.emit(self.id(), result["message"])
            return

        updated = False
        nb_adapters_changed = False
        for name, value in result.items():
            if name in self._settings and self._settings[name] != value:
                log.info("{}: updating {} from '{}' to '{}'".format(self.name(), name, self._settings[name], value))
                updated = True
                if name == "name":
                    # update the node name
                    self.updateAllocatedName(value)
                if name == "adapters":
                    nb_adapters_changed = True
                self._settings[name] = value

        if nb_adapters_changed:
            log.debug("number of adapters has changed to {}".format(self._settings["adapters"]))
            # TODO: dynamically add/remove adapters
            self._ports.clear()
            self._addAdapters(self._settings["adapters"])

        if updated:
            log.info("QEMU VM {} has been updated".format(self.name()))
            self.updated_signal.emit()

    def suspend(self):
        """
        Suspends this QEMU VM instance.
        """

        if self.status() == Node.suspended:
            log.debug("{} is already suspended".format(self.name()))
            return

        log.debug("{} is being suspended".format(self.name()))
        self.httpPost("/qemu/vms/{vm_id}/suspend".format(vm_id=self._vm_id), self._suspendCallback)

    def _suspendCallback(self, result, error=False, **kwargs):
        """
        Callback for suspend.

        :param result: server response
        :param error: indicates an error (boolean)
        """

        if error:
            log.error("error while suspending {}: {}".format(self.name(), result["message"]))
            self.server_error_signal.emit(self.id(), result["message"])
        else:
            log.info("{} has suspended".format(self.name()))
            self.setStatus(Node.suspended)
            for port in self._ports:
                # set ports as suspended
                port.setStatus(Port.suspended)
            self.suspended_signal.emit()

    def reload(self):
        """
        Reloads this QEMU VM instance.
        """

        log.debug("{} is being reloaded".format(self.name()))
        self.httpPost("/qemu/vms/{vm_id}/reload".format(vm_id=self._vm_id), self._reloadCallback)

    def _reloadCallback(self, result, error=False, **kwargs):
        """
        Callback for reload.

        :param result: server response
        :param error: indicates an error (boolean)
        """

        if error:
            log.error("error while reloading {}: {}".format(self.name(), result["message"]))
            self.server_error_signal.emit(self.id(), result["message"])
        else:
            log.info("{} has reloaded".format(self.name()))

    def dump(self):
        """
        Returns a representation of this QEMU VM instance.
        (to be saved in a topology file).

        :returns: representation of the node (dictionary)
        """

        qemu_vm = {"id": self.id(),
                   "vm_id": self._vm_id,
                   "linked_clone": self._linked_clone,
                   "type": self.__class__.__name__,
                   "description": str(self),
                   "properties": {},
                   "port_name_format": self._port_name_format,
                   "server_id": self._server.id()}

        if self._port_segment_size:
            qemu_vm["port_segment_size"] = self._port_segment_size
        if self._first_port_name:
            qemu_vm["first_port_name"] = self._first_port_name

        # add the properties
        for name, value in self._settings.items():
            if value is not None and value != "":
                qemu_vm["properties"][name] = value

        # add the ports
        if self._ports:
            ports = qemu_vm["ports"] = []
            for port in self._ports:
                ports.append(port.dump())

        return qemu_vm

    def info(self):
        """
        Returns information about this QEMU VM instance.

        :returns: formated string
        """

        if self.status() == Node.started:
            state = "started"
        else:
            state = "stopped"

        info = """QEMU VM {name} is {state}
  Node ID is {id}, server's QEMU VM ID is {vm_id}
  QEMU VM's server runs on {host}:{port}
  Console is on port {console} and type is {console_type}
""".format(name=self.name(),
           id=self.id(),
           vm_id=self._vm_id,
           state=state,
           host=self._server.host(),
           port=self._server.port(),
           console=self._settings["console"],
           console_type=self._settings["console_type"])

        port_info = ""
        for port in self._ports:
            if port.isFree():
                port_info += "     {port_name} is empty\n".format(port_name=port.name())
            else:
                port_info += "     {port_name} {port_description}\n".format(port_name=port.name(),
                                                                            port_description=port.description())

        if "usage" in self._settings and len(self._settings["usage"]) > 0:
            info += "  Usage: {}\n".format(self._settings["usage"])

        return info + port_info

    def load(self, node_info):
        """
        Loads a QEMU VM representation
        (from a topology file).

        :param node_info: representation of the node (dictionary)
        """

        # for backward compatibility
        vm_id = node_info.get("qemu_id")
        if not vm_id:
            vm_id = node_info.get("vm_id")
        linked_clone = node_info.get("linked_clone", True)
        port_name_format = node_info.get("port_name_format", "Ethernet{0}")
        port_segment_size = node_info.get("port_segment_size", 0)
        first_port_name = node_info.get("first_port_name", "")

        # prepare the VM settings
        vm_settings = {}
        for name, value in node_info["properties"].items():
            if name in self._settings:
                vm_settings[name] = value
        name = vm_settings.pop("name")
        qemu_path = vm_settings.pop("qemu_path")
        log.info("QEMU VM {} is loading".format(name))
        self.setName(name)
        self._loading = True
        self._node_info = node_info
        self.loaded_signal.connect(self._updatePortSettings)
        self.setup(qemu_path, name, vm_id, port_name_format, port_segment_size, first_port_name, linked_clone, vm_settings)

    def _updatePortSettings(self):
        """
        Updates port settings when loading a topology.
        """

        self.loaded_signal.disconnect(self._updatePortSettings)

        # assign the correct names and IDs to the ports
        if "ports" in self._node_info:
            ports = self._node_info["ports"]
            for topology_port in ports:
                for port in self._ports:
                    adapter_number = topology_port.get("adapter_number", topology_port["port_number"])
                    if adapter_number == port.adapterNumber():
                        port.setName(topology_port["name"])
                        port.setId(topology_port["id"])

        # now we can set the node as initialized and trigger the created signal
        self.setInitialized(True)
        log.info("QEMU VM {} has been loaded".format(self.name()))
        self.created_signal.emit(self.id())
        self._module.addNode(self)
        self._loading = False
        self._node_info = None

    def name(self):
        """
        Returns the name of this QEMU VM instance.

        :returns: name (string)
        """

        return self._settings["name"]

    def settings(self):
        """
        Returns all this QEMU VM instance settings.

        :returns: settings dictionary
        """

        return self._settings

    def ports(self):
        """
        Returns all the ports for this QEMU VM instance.

        :returns: list of Port instances
        """

        return self._ports

    def console(self):
        """
        Returns the console port for this QEMU VM instance.

        :returns: port (integer)
        """

        return self._settings["console"]

    def configPage(self):
        """
        Returns the configuration page widget to be used by the node properties dialog.

        :returns: QWidget object
        """

        from .pages.qemu_vm_configuration_page import QemuVMConfigurationPage
        return QemuVMConfigurationPage

    @staticmethod
    def defaultSymbol():
        """
        Returns the default symbol path for this node.

        :returns: symbol path (or resource).
        """

        return ":/symbols/qemu_guest.svg"

    @staticmethod
    def symbolName():

        return "QEMU VM"

    @staticmethod
    def categories():
        """
        Returns the node categories the node is part of (used by the device panel).

        :returns: list of node category (integer)
        """

        return [Node.end_devices]

    def __str__(self):

        return "QEMU VM"



# -*- coding: utf-8 -*-
#
# Copyright (C) 2015 GNS3 Technologies Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""
Configuration page for VMware preferences.
"""

import os
import sys
import shutil
from gns3.qt import QtWidgets
from gns3.utils.sudo import sudo

from .. import VMware
from ..ui.vmware_preferences_page_ui import Ui_VMwarePreferencesPageWidget
from ..settings import VMWARE_SETTINGS


class VMwarePreferencesPage(QtWidgets.QWidget, Ui_VMwarePreferencesPageWidget):

    """
    QWidget preference page for VMware.
    """

    def __init__(self):

        super().__init__()
        self.setupUi(self)

        # connect signals
        self.uiUseLocalServercheckBox.stateChanged.connect(self._useLocalServerSlot)
        self.uiRestoreDefaultsPushButton.clicked.connect(self._restoreDefaultsSlot)
        self.uiVmrunPathToolButton.clicked.connect(self._vmrunPathBrowserSlot)
        self.uiConfigureVmnetPushButton.clicked.connect(self._configureVmnetSlot)
        self.uiResetVmnetPushButton.clicked.connect(self._resetVmnetSlot)

        if sys.platform.startswith("darwin"):
            self.uiHostTypeComboBox.addItem("VMware Fusion", "fusion")
        else:
            self.uiHostTypeComboBox.addItem("VMware Player", "player")
            self.uiHostTypeComboBox.addItem("VMware Workstation", "ws")

        if sys.platform.startswith("win"):
            # VMnet limit on Windows is 19
            self.uiVMnetEndRangeSpinBox.setMaximum(19)
        else:
            # VMnet limit on Linux is 255
            self.uiVMnetEndRangeSpinBox.setMaximum(255)

    def _vmrunPathBrowserSlot(self):
        """
        Slot to open a file browser and select vmrun.
        """

        vmrun_path = shutil.which("vmrun")
        path, _ = QtWidgets.QFileDialog.getOpenFileName(self, "Select vmrun", vmrun_path)
        if not path:
            return

        if self._checkVmrunPath(path):
            self.uiVmrunPathLineEdit.setText(os.path.normpath(path))

    def _checkVmrunPath(self, path):
        """
        Checks that the vmrun path is valid.

        :param path: vmrun path
        :returns: boolean
        """

        if not os.path.exists(path):
            QtWidgets.QMessageBox.critical(self, "vmrun", '"{}" does not exist'.format(path))
            return False

        if not os.access(path, os.X_OK):
            QtWidgets.QMessageBox.critical(self, "vmrun", "{} is not an executable".format(os.path.basename(path)))
            return False

        return True

    def _restoreDefaultsSlot(self):
        """
        Slot to populate the page widgets with the default settings.
        """

        self._populateWidgets(VMWARE_SETTINGS)

    def _useLocalServerSlot(self, state):
        """
        Slot to enable or not local server settings.
        """

        if state:
            self.uiVmrunPathLineEdit.setEnabled(True)
            self.uiVmrunPathToolButton.setEnabled(True)
            self.uiHostTypeComboBox.setEnabled(True)
            self.uiNetworkTab.setEnabled(True)
        else:
            self.uiVmrunPathLineEdit.setEnabled(False)
            self.uiVmrunPathToolButton.setEnabled(False)
            self.uiHostTypeComboBox.setEnabled(False)
            self.uiNetworkTab.setEnabled(False)

    def _getGNS3Vmnet(self):
        """
        Get the gns3vmnet utility path.
        """

        gns3vmnet = shutil.which("gns3vmnet")
        if gns3vmnet is None:
            QtWidgets.QMessageBox.critical(self, "gns3vmnet", "The gns3vmnet utility is not installed")
            return None
        return gns3vmnet

    def _configureVmnetSlot(self):
        """
        Configure the vmnet interfaces.
        """

        vmnet_start = str(self.uiVMnetStartRangeSpinBox.value())
        vmnet_end = str(self.uiVMnetEndRangeSpinBox.value())
        gns3vmnet = self._getGNS3Vmnet()
        if gns3vmnet is None:
            return
        command = [gns3vmnet, "-r", vmnet_start, vmnet_end]
        sudo(command, parent=self)

    def _resetVmnetSlot(self):
        """
        Deletes all vmnet interface but vmnet1 and vmnet8
        """

        gns3vmnet = self._getGNS3Vmnet()
        if gns3vmnet is None:
            return
        command = [gns3vmnet, "-C"]
        sudo(command, parent=self)

    def _populateWidgets(self, settings):
        """
        Populates the widgets with the settings.

        :param settings: VMware settings
        """

        self.uiVmrunPathLineEdit.setText(settings["vmrun_path"])
        index = self.uiHostTypeComboBox.findData(settings["host_type"])
        if index != -1:
            self.uiHostTypeComboBox.setCurrentIndex(index)
        self.uiVMnetStartRangeSpinBox.setValue(settings["vmnet_start_range"])
        self.uiVMnetEndRangeSpinBox.setValue(settings["vmnet_end_range"])
        self.uiUseLocalServercheckBox.setChecked(settings["use_local_server"])

    def loadPreferences(self):
        """
        Loads VMware preferences.
        """

        vmware_settings = VMware.instance().settings()
        self._populateWidgets(vmware_settings)

    def savePreferences(self):
        """
        Saves VMware preferences.
        """

        vmrun_path = self.uiVmrunPathLineEdit.text().strip()
        if vmrun_path and self.uiUseLocalServercheckBox.isChecked() and not self._checkVmrunPath(vmrun_path):
            return

        new_settings = {"vmrun_path": vmrun_path,
                        "host_type": self.uiHostTypeComboBox.itemData(self.uiHostTypeComboBox.currentIndex()),
                        "vmnet_start_range": self.uiVMnetStartRangeSpinBox.value(),
                        "vmnet_end_range": self.uiVMnetEndRangeSpinBox.value(),
                        "use_local_server": self.uiUseLocalServercheckBox.isChecked()}
        VMware.instance().setSettings(new_settings)



# -*- coding: utf-8 -*-
#
# Copyright (C) 2015 GNS3 Technologies Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""
Default VMware settings.
"""

import sys
from gns3.node import Node

if sys.platform.startswith("win"):
    DEFAULT_VMNET_END_RANGE = 19
else:
    DEFAULT_VMNET_END_RANGE = 100

VMWARE_SETTINGS = {
    "vmrun_path": "",
    "host_type": "ws",
    "vmnet_start_range": 2,
    "vmnet_end_range": DEFAULT_VMNET_END_RANGE,
    "use_local_server": True
}

VMWARE_VM_SETTINGS = {
    "vmx_path": "",
    "symbol": ":/symbols/vmware_guest.svg",
    "category": Node.end_devices,
    "port_name_format": "Ethernet{0}",
    "port_segment_size": 0,
    "first_port_name": "",
    "adapters": 1,
    "adapter_type": "default",
    "use_ubridge": True,
    "use_any_adapter": False,
    "headless": False,
    "acpi_shutdown": False,
    "enable_remote_console": False,
    "linked_base": False,
    "server": "local"
}



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file '/home/grossmj/PycharmProjects/gns3-gui/gns3/ui/export_debug_dialog.ui'
#
# Created: Sat Nov 14 18:32:47 2015
#      by: PyQt5 UI code generator 5.2.1
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_ExportDebugDialog(object):
    def setupUi(self, ExportDebugDialog):
        ExportDebugDialog.setObjectName("ExportDebugDialog")
        ExportDebugDialog.setWindowModality(QtCore.Qt.WindowModal)
        ExportDebugDialog.resize(555, 215)
        ExportDebugDialog.setModal(True)
        self.verticalLayout = QtWidgets.QVBoxLayout(ExportDebugDialog)
        self.verticalLayout.setContentsMargins(-1, -1, 12, -1)
        self.verticalLayout.setObjectName("verticalLayout")
        self.label = QtWidgets.QLabel(ExportDebugDialog)
        self.label.setTextFormat(QtCore.Qt.RichText)
        self.label.setScaledContents(False)
        self.label.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignTop)
        self.label.setWordWrap(True)
        self.label.setObjectName("label")
        self.verticalLayout.addWidget(self.label)
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setSpacing(20)
        self.horizontalLayout.setObjectName("horizontalLayout")
        spacerItem = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem)
        self.uiCancelButton = QtWidgets.QPushButton(ExportDebugDialog)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiCancelButton.sizePolicy().hasHeightForWidth())
        self.uiCancelButton.setSizePolicy(sizePolicy)
        self.uiCancelButton.setObjectName("uiCancelButton")
        self.horizontalLayout.addWidget(self.uiCancelButton)
        self.uiOkButton = QtWidgets.QDialogButtonBox(ExportDebugDialog)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiOkButton.sizePolicy().hasHeightForWidth())
        self.uiOkButton.setSizePolicy(sizePolicy)
        self.uiOkButton.setOrientation(QtCore.Qt.Horizontal)
        self.uiOkButton.setStandardButtons(QtWidgets.QDialogButtonBox.Ok)
        self.uiOkButton.setObjectName("uiOkButton")
        self.horizontalLayout.addWidget(self.uiOkButton)
        self.verticalLayout.addLayout(self.horizontalLayout)

        self.retranslateUi(ExportDebugDialog)
        self.uiCancelButton.clicked.connect(ExportDebugDialog.reject)
        QtCore.QMetaObject.connectSlotsByName(ExportDebugDialog)

    def retranslateUi(self, ExportDebugDialog):
        _translate = QtCore.QCoreApplication.translate
        ExportDebugDialog.setWindowTitle(_translate("ExportDebugDialog", "Export debug information"))
        self.label.setText(_translate("ExportDebugDialog", "<html><head/><body><p>This will export a debug information file. You must first<span style=\" font-weight:600;\"> save a project before</span> you are allowed to export the debug data.</p><p><span style=\" font-weight:600;\">Be aware</span> this file can contain <span style=\" font-weight:600;\">private information</span> about your project, your GNS3 settings or your computer (list of running processes, opened ports etc.). You can unzip the file in order to edit its content.</p><p><br/>Thanks a lot for helping GNS3.</p></body></html>"))
        self.uiCancelButton.setText(_translate("ExportDebugDialog", "Cancel"))

from . import resources_rc



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file '/Users/noplay/code/gns3/gns3-gui/gns3/ui/new_project_dialog.ui'
#
# Created by: PyQt5 UI code generator 5.4.2
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_NewProjectDialog(object):

    def setupUi(self, NewProjectDialog):
        NewProjectDialog.setObjectName("NewProjectDialog")
        NewProjectDialog.setWindowModality(QtCore.Qt.ApplicationModal)
        NewProjectDialog.resize(577, 188)
        NewProjectDialog.setModal(True)
        self.gridLayout_2 = QtWidgets.QGridLayout(NewProjectDialog)
        self.gridLayout_2.setSizeConstraint(QtWidgets.QLayout.SetFixedSize)
        self.gridLayout_2.setObjectName("gridLayout_2")
        self.uiButtonBox = QtWidgets.QDialogButtonBox(NewProjectDialog)
        self.uiButtonBox.setOrientation(QtCore.Qt.Horizontal)
        self.uiButtonBox.setStandardButtons(QtWidgets.QDialogButtonBox.Cancel | QtWidgets.QDialogButtonBox.Ok)
        self.uiButtonBox.setObjectName("uiButtonBox")
        self.gridLayout_2.addWidget(self.uiButtonBox, 2, 2, 1, 1)
        self.uiOpenProjectPushButton = QtWidgets.QPushButton(NewProjectDialog)
        self.uiOpenProjectPushButton.setObjectName("uiOpenProjectPushButton")
        self.gridLayout_2.addWidget(self.uiOpenProjectPushButton, 2, 0, 1, 1)
        self.uiRecentProjectsPushButton = QtWidgets.QPushButton(NewProjectDialog)
        self.uiRecentProjectsPushButton.setObjectName("uiRecentProjectsPushButton")
        self.gridLayout_2.addWidget(self.uiRecentProjectsPushButton, 2, 1, 1, 1)
        self.uiProjectGroupBox = QtWidgets.QGroupBox(NewProjectDialog)
        self.uiProjectGroupBox.setObjectName("uiProjectGroupBox")
        self.gridLayout = QtWidgets.QGridLayout(self.uiProjectGroupBox)
        self.gridLayout.setObjectName("gridLayout")
        self.uiNameLabel = QtWidgets.QLabel(self.uiProjectGroupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiNameLabel.sizePolicy().hasHeightForWidth())
        self.uiNameLabel.setSizePolicy(sizePolicy)
        self.uiNameLabel.setObjectName("uiNameLabel")
        self.gridLayout.addWidget(self.uiNameLabel, 0, 0, 1, 1)
        self.uiNameLineEdit = QtWidgets.QLineEdit(self.uiProjectGroupBox)
        self.uiNameLineEdit.setObjectName("uiNameLineEdit")
        self.gridLayout.addWidget(self.uiNameLineEdit, 0, 1, 1, 3)
        self.uiLocationLabel = QtWidgets.QLabel(self.uiProjectGroupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiLocationLabel.sizePolicy().hasHeightForWidth())
        self.uiLocationLabel.setSizePolicy(sizePolicy)
        self.uiLocationLabel.setObjectName("uiLocationLabel")
        self.gridLayout.addWidget(self.uiLocationLabel, 1, 0, 1, 1)
        self.uiLocationLineEdit = QtWidgets.QLineEdit(self.uiProjectGroupBox)
        self.uiLocationLineEdit.setObjectName("uiLocationLineEdit")
        self.gridLayout.addWidget(self.uiLocationLineEdit, 1, 1, 1, 2)
        self.uiLocationBrowserToolButton = QtWidgets.QToolButton(self.uiProjectGroupBox)
        self.uiLocationBrowserToolButton.setObjectName("uiLocationBrowserToolButton")
        self.gridLayout.addWidget(self.uiLocationBrowserToolButton, 1, 3, 1, 1)
        self.gridLayout_2.addWidget(self.uiProjectGroupBox, 0, 0, 1, 4)

        self.retranslateUi(NewProjectDialog)
        self.uiButtonBox.accepted.connect(NewProjectDialog.accept)
        self.uiButtonBox.rejected.connect(NewProjectDialog.reject)
        QtCore.QMetaObject.connectSlotsByName(NewProjectDialog)

    def retranslateUi(self, NewProjectDialog):
        _translate = QtCore.QCoreApplication.translate
        NewProjectDialog.setWindowTitle(_translate("NewProjectDialog", "New project"))
        self.uiOpenProjectPushButton.setText(_translate("NewProjectDialog", "&Open a project"))
        self.uiRecentProjectsPushButton.setText(_translate("NewProjectDialog", "&Recent projects..."))
        self.uiProjectGroupBox.setTitle(_translate("NewProjectDialog", "Project"))
        self.uiNameLabel.setText(_translate("NewProjectDialog", "Name:"))
        self.uiLocationLabel.setText(_translate("NewProjectDialog", "Location:"))
        self.uiLocationBrowserToolButton.setText(_translate("NewProjectDialog", "Browse..."))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (C) 2015 GNS3 Technologies Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.


import json
import sys
import os
import urllib

from .image import Image

import logging
log = logging.getLogger(__name__)


class ConfigException(Exception):
    pass


class Config:
    """
    GNS3 config file
    """

    def __init__(self, path=None):
        """
        :params path: Path of the configuration file, otherwise detect it on the system
        """

        self.path = path
        if self.path is None:
            self.path = self._get_standard_config_file_path()

        with open(self.path, encoding="utf-8") as f:
            self._config = json.load(f)

    @property
    def images_dir(self):
        """
        :returns: Location of the images directory on the server
        """
        return self._config["Servers"]["local_server"]["images_path"]

    @property
    def symbols_dir(self):
        """
        :returns: Location of the symbols directory
        """
        return self._config["Servers"]["local_server"]["symbols_path"]

    @property
    def servers(self):
        """
        :returns: List of server present in the configuration file as strings
        """

        servers = ["local"]
        if "vm" in self._config["Servers"] and self._config["Servers"]["vm"].get("auto_start", False):
            servers.append("vm")
        if "remote_servers" in self._config["Servers"]:
            for server in self._config["Servers"]["remote_servers"]:
                if "url" in server:
                    servers.append(server["url"])
        return servers

    def _get_standard_config_file_path(self):

        if sys.platform.startswith("win"):
            filename = "gns3_gui.ini"
        else:
            filename = "gns3_gui.conf"

        appname = "GNS3"

        if sys.platform.startswith("win"):
            appdata = os.path.expandvars("%APPDATA%")
            return os.path.join(appdata, appname, filename)
        else:
            home = os.path.expanduser("~")
            return os.path.join(home, ".config", appname, filename)

    def is_name_available(self, name):
        """
        :param name: Appliance name
        :returns: True if name is not already used
        """
        for item in self._config["Qemu"].get("vms", []):
            if item["name"] == name:
                return False
        return True

    def add_appliance(self, appliance_config, server):
        """
        Add appliance to the user configuration

        :param appliance_config: Dictionary with appliance configuration
        :param server
        """

        new_config = {
            "server": server,
            "name": appliance_config["name"]
        }

        if "usage" in appliance_config:
            new_config["usage"] = appliance_config["usage"]

        if appliance_config["category"] == "guest":
            new_config["category"] = 2
        elif appliance_config["category"] == "router":
            new_config["category"] = 0
        elif appliance_config["category"] == "firewall":
            new_config["category"] = 3
        elif appliance_config["category"] == "switch":
            new_config["category"] = 1
        elif appliance_config["category"] == "multilayer_switch":
            new_config["category"] = 1

        if "symbol" in appliance_config:
            new_config["symbol"] = self._set_symbol(appliance_config["symbol"])

        if new_config.get("symbol") is None:
            if appliance_config["category"] == "guest":
                new_config["symbol"] = ":/symbols/qemu_guest.svg"
            elif appliance_config["category"] == "router":
                new_config["symbol"] = ":/symbols/router.svg"
            elif appliance_config["category"] == "switch":
                new_config["symbol"] = ":/symbols/ethernet_switch.svg"
            elif appliance_config["category"] == "multilayer_switch":
                new_config["symbol"] = ":/symbols/multilayer_switch.svg"
            elif appliance_config["category"] == "firewall":
                new_config["symbol"] = ":/symbols/firewall.svg"

        # Raise error if VM already exists
        if not self.is_name_available(new_config["name"]):
            raise ConfigException("{} already exists".format(new_config["name"]))

        if "qemu" in appliance_config:
            self._add_qemu_config(new_config, appliance_config)
            return
        if "iou" in appliance_config:
            self._add_iou_config(new_config, appliance_config)
            return
        if "dynamips" in appliance_config:
            self._add_dynamips_config(new_config, appliance_config)
            return
        raise ConfigException("{} no configuration found for know emulators".format(new_config["name"]))

    def _add_dynamips_config(self, new_config, appliance_config):
        new_config["auto_delete_disks"] = True
        new_config["disk0"] = 0
        new_config["disk1"] = 0
        new_config["exec_area"] = 64
        new_config["idlemax"] = 500
        new_config["idlesleep"] = 30
        new_config["system_id"] = "FTX0945W0MY"
        new_config["sparsemem"] = True
        new_config["private_config"] = ""
        new_config["mac_addr"] = ""
        new_config["iomem"] = 5
        new_config["mmap"] = True

        for key, value in appliance_config["dynamips"].items():
            new_config[key] = value

        for image in appliance_config["images"]:
            new_config[image["type"]] = self._relative_image_path("IOS", image["filename"], image["path"])
            new_config["idlepc"] = image["idlepc"]

        log.debug("Add appliance Dynamips: %s", str(new_config))
        self._config["Dynamips"]["routers"].append(new_config)

    def _add_iou_config(self, new_config, appliance_config):
        new_config["ethernet_adapters"] = appliance_config["iou"]["ethernet_adapters"]
        new_config["serial_adapters"] = appliance_config["iou"]["serial_adapters"]
        new_config["startup_config"] = appliance_config["iou"]["startup_config"]
        new_config["private_config"] = ""
        new_config["l1_keepalives"] = False
        new_config["use_default_iou_values"] = True
        new_config["nvram"] = appliance_config["iou"]["nvram"]
        new_config["ram"] = appliance_config["iou"]["ram"]

        for image in appliance_config["images"]:
            new_config[image["type"]] = self._relative_image_path("IOU", image["filename"], image["path"])
        new_config["path"] = new_config["image"]

        log.debug("Add appliance IOU: %s", str(new_config))
        self._config["IOU"]["devices"].append(new_config)

    def _add_qemu_config(self, new_config, appliance_config):

        new_config["adapter_type"] = appliance_config["qemu"]["adapter_type"]
        new_config["adapters"] = appliance_config["qemu"]["adapters"]
        new_config["cpu_throttling"] = appliance_config["qemu"].get("cpu_throttling", 0)
        new_config["ram"] = appliance_config["qemu"]["ram"]
        new_config["console_type"] = appliance_config["qemu"]["console_type"]
        new_config["legacy_networking"] = False
        new_config["process_priority"] = appliance_config["qemu"].get("process_priority", "normal")

        options = appliance_config["qemu"].get("options", "")
        if "-nographic" not in options:
            options += " -nographic"
        if appliance_config["qemu"].get("kvm", "allow") == "disable" and "-no-kvm" not in options:
            options += " -no-kvm"
        new_config["options"] = options.strip()

        for image in appliance_config["images"]:
            new_config[image["type"]] = self._relative_image_path("QEMU", image["filename"], image["path"])

        new_config.setdefault("hda_disk_image", "")
        new_config.setdefault("hdb_disk_image", "")
        new_config.setdefault("hdc_disk_image", "")
        new_config.setdefault("hdd_disk_image", "")
        new_config.setdefault("cdrom_image", "")
        new_config.setdefault("initrd", "")
        new_config.setdefault("kernel_image", "")

        new_config["hda_disk_interface"] = appliance_config["qemu"].get("hda_disk_interface", "ide")
        new_config["hdb_disk_interface"] = appliance_config["qemu"].get("hdb_disk_interface", "ide")
        new_config["hdc_disk_interface"] = appliance_config["qemu"].get("hdc_disk_interface", "ide")
        new_config["hdd_disk_interface"] = appliance_config["qemu"].get("hdd_disk_interface", "ide")

        new_config["kernel_command_line"] = appliance_config["qemu"].get("kernel_command_line", "")

        if "path" in appliance_config["qemu"]:
            new_config["qemu_path"] = appliance_config["qemu"]["path"]
        else:
            new_config["qemu_path"] = "qemu-system-{}".format(appliance_config["qemu"]["arch"])

        if "boot_priority" in appliance_config:
            new_config["boot_priority"] = appliance_config["boot_priority"]

        if "first_port_name" in appliance_config:
            new_config["first_port_name"] = appliance_config["first_port_name"]

        if "port_name_format" in appliance_config:
            new_config["port_name_format"] = appliance_config["port_name_format"]

        if "port_segment_size" in appliance_config:
            new_config["port_segment_size"] = appliance_config["port_segment_size"]

        if "linked_base" in appliance_config:
            new_config["linked_base"] = appliance_config["linked_base"]

        log.debug("Add appliance QEMU: %s", str(new_config))
        self._config["Qemu"].setdefault("vms", [])
        self._config["Qemu"]["vms"].append(new_config)

    def _set_symbol(self, symbol):
        """
        Download symbol for the web if need
        """

        # GNS3 builtin symbol
        if symbol.startswith(":/symbols/"):
            return symbol

        path = os.path.join(self.symbols_dir, symbol)
        if os.path.exists(path):
            return path

        url = "https://raw.githubusercontent.com/GNS3/gns3-registry/master/symbols/{}".format(symbol)
        try:
            urllib.request.urlretrieve(url, path)
            return path
        except OSError:
            return None

    def _relative_image_path(self, image_dir_type, filename, path):
        """
        :param image_dir_type: Type of image directory
        :param filename: Filename at the end of the processus
        :param path: Full path to the file
        :returns: Path relative to image directory.
        Copy the image to the directory if not already in the directory
        """

        images_dir = os.path.join(self.images_dir, image_dir_type)
        path = os.path.abspath(path)
        if os.path.commonprefix([images_dir, path]) == images_dir:
            return path.replace(images_dir, '').strip('/\\')

        Image(path).copy(images_dir, filename)
        return filename

    def save(self):
        """
        Save the configuration file
        """

        with open(self.path, "w+", encoding="utf-8") as f:
            json.dump(self._config, f, indent=4)



#!/usr/bin/env python
#
# Copyright (C) 2015 GNS3 Technologies Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.


import sys
import shutil
import json
import os
from datetime import datetime


try:
    from gns3.qt import QtGui, QtWidgets
except ImportError:
    raise SystemExit("Can't import Qt modules: Qt and/or PyQt is probably not installed correctly...")

import logging
log = logging.getLogger(__name__)


from gns3.version import __version__
from gns3.local_config import LocalConfig
from gns3.ui.iouvm_converter_wizard_ui import Ui_IOUVMConverterWizard


class IOUVMConverterWizard(QtWidgets.QWizard, Ui_IOUVMConverterWizard):

    def __init__(self, parent=None):
        super().__init__(parent)
        self.setupUi(self)
        self.setWizardStyle(QtWidgets.QWizard.ModernStyle)
        if sys.platform.startswith("darwin"):
            # we want to see the cancel button on OSX
            self.setOptions(QtWidgets.QWizard.NoDefaultButton)

        # set the window icon
        self.setWindowIcon(QtGui.QIcon(":/images/gns3.ico"))  # this info is necessary for QSettings

        config = self._loadConfig()
        self.uiPushButtonBrowse.clicked.connect(self._browseTopologiesSlot)
        self.uiLineEditTopologiesPath.setText(config['Servers']['local_server']['projects_path'])

    def _browseTopologiesSlot(self):
        path = QtWidgets.QFileDialog.getExistingDirectory(self, 'Select a directory')
        self.uiLineEditTopologiesPath.setText(path)

    def validateCurrentPage(self):
        """
        Validates the settings.
        """

        if self.currentPage() == self.uiWizardPageIOURCCheck:
            return self._checkIOURC()
        elif self.currentPage() == self.uiWizardUpdateConfiguration:
            return self._updateConfig()
        elif self.currentPage() == self.uiWizardPagePatchTopologies:
            return self._patchTopologies()
        return True

    def _checkIOURC(self):
        """
        Validate if the IOURC contain an entry for the IOUVM
        """
        config = self._loadConfig()
        iourc_path = config.get("IOU", {}).get("iourc_path", "")
        if len(iourc_path) == 0:
            QtWidgets.QMessageBox.critical(self, "Error", "The IOURC is not configured")
            return False
        try:
            with open(iourc_path) as f:
                if 'gns3vm' not in f.read():
                    QtWidgets.QMessageBox.critical(self, "Error", "The gns3vm doesn't exist in your iourc file".format(iourc_path))
        except OSError:
            QtWidgets.QMessageBox.critical(self, "Error", "IOURC file {} doesn't exist or not accessible".format(iourc_path))
        return True

    def _updateConfig(self):
        """
        Update the config file to use the GNS3 VM instead of IOU VM
        """
        config = self._loadConfig()
        if "devices" in config["IOU"]:
            for device in config["IOU"]["devices"]:
                device["path"] = os.path.basename(device["path"])
                device["server"] = "vm"
        config["Servers"]["remote_servers"] = []
        self._writeConfig(config)
        return True

    def _patchTopologies(self):
        """
        Patch topologies to use the GNS3 VM
        """

        path = self.uiLineEditTopologiesPath.text()
        try:
            for (dirpath, dirnames, filenames) in os.walk(path):
                for filename in filenames:
                    if filename.endswith(".gns3"):
                        self._patchTopology(os.path.join(dirpath, filename))
        except OSError as e:
            QtWidgets.QMessageBox.critical(self, "Error", "Can't open {}: {}".format(path, str(e)))
            return False
        return True

    def _patchTopology(self, path):
        """
        Path a specific topology
        """
        try:
            shutil.copy(path, "{}.{}.backup".format(path, datetime.now().isoformat()))
            with open(path) as f:
                topo = json.load(f)
            if "topology" in topo and "servers" in topo["topology"]:
                for server in topo["topology"]["servers"]:
                    if server["local"] is False:
                        server["vm"] = True
                with open(path, 'w+') as f:
                    topo = json.dump(topo, f)
        except OSError as e:
            QtWidgets.QMessageBox.critical(self, "Error", "Can't open {}: {}".format(path, str(e)))

    def _loadConfig(self):
        with open(self._configurationFile()) as f:
            return json.load(f)

    def _writeConfig(self, config):
        shutil.copy(self._configurationFile(), "{}.{}.backup".format(self._configurationFile(), datetime.now().isoformat()))
        with open(self._configurationFile(), 'w+') as f:
            json.dump(config, f, indent=4)

    def _configurationFile(self):
        if sys.platform.startswith("win"):
            filename = "gns3_gui.ini"
        else:
            filename = "gns3_gui.conf"
        directory = LocalConfig.configDirectory()
        return os.path.join(directory, filename)


def main():
    app = QtWidgets.QApplication(sys.argv)

    app.setOrganizationName("GNS3")
    app.setOrganizationDomain("gns3.net")
    app.setApplicationName("GNS3")
    app.setApplicationVersion(__version__)

    # We force a full garbage collect before exit
    # for unknow reason otherwise Qt Segfault on OSX in some
    # conditions
    import gc
    gc.collect()

    # Manage Ctrl + C or kill command
    def sigint_handler(*args):
        log.info("Signal received exiting the application")
        app.closeAllWindows()
    # signal.signal(signal.SIGINT, sigint_handler)
    # signal.signal(signal.SIGTERM, sigint_handler)

    mainwindow = IOUVMConverterWizard()
    mainwindow.show()
    exit_code = mainwindow.exec_()

    # We force a full garbage collect before exit
    # for unknow reason otherwise Qt Segfault on OSX in some
    # conditions
    import gc
    gc.collect()

    sys.exit(exit_code)

if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-
#
# Copyright (C) 2014 GNS3 Technologies Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""
Compatibility layer for Qt bindings, so it is easier to switch from PyQt4 to PyQt5 and
vice-versa. It is possible to add PySide if needed.
For PyQt4 and PyQt5 differences please see http://pyqt.sourceforge.net/Docs/PyQt5/pyqt4_differences.html
"""

# based on https://gist.github.com/remram44/5985681 and
# https://github.com/pyQode/pyqode.qt/blob/master/pyqode/qt/QtWidgets.py (MIT license)


import sys
import sip
import os
import functools
import inspect

import logging
log = logging.getLogger(__name__)


if os.environ.get('GNS3_QT4', None) is not None:
    DEFAULT_BINDING = 'PyQt4'
else:
    try:
        import PyQt5
        DEFAULT_BINDING = 'PyQt5'
    except ImportError:
        DEFAULT_BINDING = 'PyQt4'


if DEFAULT_BINDING == 'PyQt4':
    log.critical('ERROR: PyQt4 is no longer supported, please upgrade to PyQt5 for Python 3')
    from PyQt4 import QtGui
    app = QtGui.QApplication(sys.argv)
    message = QtGui.QMessageBox.critical(None, 'GNS3', 'PyQt4 is no longer supported, please upgrade to PyQt5 for Python 3')
    sys.exit(1)


from PyQt5 import QtCore, QtGui, QtNetwork, QtWidgets, Qt
sys.modules[__name__ + '.QtCore'] = QtCore
sys.modules[__name__ + '.QtGui'] = QtGui
sys.modules[__name__ + '.QtNetwork'] = QtNetwork
sys.modules[__name__ + '.QtWidgets'] = QtWidgets

try:
    from PyQt5 import QtSvg
    sys.modules[__name__ + '.QtSvg'] = QtSvg
except ImportError:
    raise SystemExit("Please install the PyQt5.QtSvg module")

QtCore.Signal = QtCore.pyqtSignal
QtCore.Slot = QtCore.pyqtSlot
QtCore.Property = QtCore.pyqtProperty
QtCore.BINDING_VERSION_STR = QtCore.PYQT_VERSION_STR

from PyQt5.QtWidgets import QFileDialog as OldFileDialog


class QFileDialog(OldFileDialog):
    @staticmethod
    def getExistingDirectory(parent=None, caption='', dir='', options=OldFileDialog.ShowDirsOnly):
        path = OldFileDialog.getExistingDirectory(parent, caption, dir, options)
        if path:
            path = os.path.normpath(path)
        return path

    @staticmethod
    def getOpenFileName(parent=None, caption='', directory='', filter='', selectedFilter='', options=OldFileDialog.Options()):
        path, _ = OldFileDialog.getOpenFileName(parent, caption, directory, filter, selectedFilter, options)
        if path:
            path = os.path.normpath(path)
        return path, _

    @staticmethod
    def getOpenFileNames(parent=None, caption='', directory='', filter='', selectedFilter='', options=OldFileDialog.Options()):
        path, _ = OldFileDialog.getOpenFileNames(parent, caption, directory, filter, selectedFilter, options)
        if path:
            path = os.path.normpath(path)
        return path, _

    @staticmethod
    def getSaveFileName(parent=None, caption='', directory='', filter='', selectedFilter='', options=OldFileDialog.Options()):
        path, _ = OldFileDialog.getSaveFileName(parent, caption, directory, filter, selectedFilter, options)
        if path:
            path = os.path.normpath(path)
        return path, _

QtWidgets.QFileDialog = QFileDialog


class StatsQtWidgetsQDialog(QtWidgets.QDialog):
    """
    Send stats from all the QWizard
    """

    def __init__(self, *args):
        super().__init__(*args)

        import re
        from .utils.analytics import AnalyticsClient
        name = self.__class__.__name__
        name = re.sub(r"([A-Z])", r" \1", name).strip()
        AnalyticsClient.instance().sendScreenView(name)

QtWidgets.QDialog = StatsQtWidgetsQDialog


class LogQMessageBox(QtWidgets.QMessageBox):
    """
    Replace the standard message box for logging errors to console. And
    show a stack trace when a critical message box is shown in debug mode
    """
    @staticmethod
    def critical(parent, title, message, *args):
        log.critical(message, stack_info=LogQMessageBox.stack_info())
        return super(QtWidgets.QMessageBox, QtWidgets.QMessageBox).critical(parent, title, message, *args)

    @staticmethod
    def warning(parent, title, message, *args):
        log.warning(message)
        return super(QtWidgets.QMessageBox, QtWidgets.QMessageBox).warning(parent, title, message, *args)


    @staticmethod
    def stack_info():
        """
        Show stack trace
        """
        return logging.getLogger().getEffectiveLevel() == logging.DEBUG

QtWidgets.QMessageBox = LogQMessageBox


class StatsQtWidgetsQWizard(QtWidgets.QWizard):
    """
    Send stats from all the QWizard
    """

    def __init__(self, *args):
        super().__init__(*args)

        import re
        from .utils.analytics import AnalyticsClient
        name = self.__class__.__name__
        name = re.sub(r"([A-Z])", r" \1", name).strip()
        AnalyticsClient.instance().sendScreenView(name)

QtWidgets.QWizard = StatsQtWidgetsQWizard


class StatsQtWidgetsQMainWindow(QtWidgets.QMainWindow):
    """
    Send stats from all the QMainWindow
    """

    def __init__(self, *args):
        super().__init__(*args)

        import re
        from .utils.analytics import AnalyticsClient
        name = self.__class__.__name__
        name = re.sub(r"([A-Z])", r" \1", name).strip()
        AnalyticsClient.instance().sendScreenView(name)

QtWidgets.QMainWindow = StatsQtWidgetsQMainWindow

# If we run from a test we replace the signal by a synchronous version
if hasattr(sys, '_called_from_test'):
    class FakeQtSignal:
        _instances = set()

        def __init__(self, *args):
            self._callbacks = set()
            self._instances.add(self)

        def connect(self, func, style=None):
            log.debug("{caller} connect to signal".format(caller=sys._getframe(1).f_code.co_name))
            self._callbacks.add(func)

        def disconnect(self, func):
            self._callbacks.remove(func)

        def emit(self, *args):
            log.debug("{caller} emit signal".format(caller=sys._getframe(1).f_code.co_name))
            for callback in list(self._callbacks):
                callback(*args)

        @classmethod
        def reset(cls):
            """Use to clean the listening signals between tests"""
            for instance in cls._instances:
                instance._callbacks = set()

    QtCore.Signal = FakeQtSignal
    QtCore.pyqtSignal = FakeQtSignal


def qpartial(func, *args, **kwargs):
    """
    A functools partial that you can use on qobject. If the targeted qobject is
    destroyed the partial is not called.
    """
    if func is None:
        return None

    if inspect.ismethod(func):
        if isinstance(func.__self__, QtCore.QObject):

            def partial(*args, **kwargs):
                if sip.isdeleted(func.__self__):
                    return
                return func(*args, **kwargs)
            return functools.partial(partial, *args, **kwargs)

    return functools.partial(func, *args, **kwargs)
